@@ -1,99 +1,18 @@
 package Config::Simple::Conf;
-# Copyright 2016 (C) Colin Faber <cfaber@fpsn.net>
-# Licensed under the terms of perl itself.
+# @@COPYRIGHT@@
 
 use strict;
 # Version number - cvs automagically updated.
-our $VERSION = $1 if('$Revision: 1.10 $' =~ /: ([\d\.]+)/);
+our $VERSION = $1 if('$Revision: 1.2 $' =~ /: ([\d\.]+)/);
 
 # Revision - cvs automagically updated
-our $REVISION = '$Id: Conf.pm,v 1.10 2016/05/24 18:33:29 cfaber Exp $';
+our $REVISION = '$Id: Conf.pm,v 1.2 2016/04/15 19:50:25 cfaber Exp $';
 
 # Ruckus Global Version Number
 
 =head1 NAME
 
-Config::Simple::Conf - A fast and lightweight configuration file handler
-
-=head1 DESCRIPTION
-
-The idea behind Config::Simple::Conf came from various INI style parsers I've used in the past. In general these have worked well with the exception of lack of complex configuration handling.
-
-Config::Simple for example fails to account for common cases which are extremely useful in any configuration file. These include useful handling of duplicate keys (currently Config::Simple blows them away without any notice), and second, internal macros.
-
-In many of my usage cases I want something like your standard .INI file format, with the above mentioned exceptions.
-
-	# Define a configuration section
-	[core section]
-
-	# Define an entry in core section
-	path = /root/to/my/stuff
-
-	# Define a new configuration file section
-	[section name]
-
-	# Define an entry list and use the value from another section to complete
-	# the configuration
-	path = [core section:path]/abc
-	path = [core section:path]/xyz
-
-Such a configuration would allow me to do two things, establish a core path argument, which is then used in other sections, and have a section with multiple duplicate entires as a list.
-
-An example of the code here would look something like:
-
-	#!/usr/bin/perl
-
-	use strict;
-	use Config::Simple::Conf;
-
-	my $conf = Config::Simple::Conf->new('/path/to/my.conf');
-
-	print "My root is: " . $conf->value('core section', 'path') . "\n";
-	print "My section paths are:\n";
-
-	for($conf->value('section name', 'path')){
-		print "\t$_\n";
-	}
-
-With the resulting output looking something like:
-
-	My root is: /root/to/my/stuff
-	My section paths are:
-		/root/to/my/stuff/abc
-		/root/to/my/stuff/xyz
-
-=head1 CONFIG FILE FORMAT
-
-Configuration files are defined as ascii text, with comments lines starting with a pound symbol B<#>, sections, keys, and values. Values may be macro entries referencing other configuration keys.
-
-=head2 SECTION
-
-A section is defined as a single line entry with double square brakets B<[section]>:
-
-	# Define a section
-	[section]
-
-=head2 KEYS
-
-Keys are defined within a section as lines with keyname = value type entry
-
-	# Define a value for keyname in section [section]
-	[section]
-	keyname = value
-
-=head2 USING A MACRO
-
-Macros are defined as square brakets with a section:key entry between them. These are automatically resolved to other configuration keys and those key values are utilized.
-
-Macros may B<NOT> utilize list entries as a macro value at this point.
-
-	# Define a value based on a macro
-	[section2]
-	key = [section:keyname]
-
-=head2 EXAMPLES
-
-See examples/ directory for various configuration file examples
+Config::Simple::Conf - an extremely fast, light weight configuration file parser with caching
 
 =head1 SYNOPSIS
 
@@ -103,6 +22,10 @@ my $conf = Config::Simple::Conf->new('/etc/Something/Example.conf');
 
 print $conf->value('global', 'example_key');
 
+=head1 CONFIGURATION FILE FORMAT
+
+See examples/ directory for various configuration file examples
+
 =head1 METHODS
 
 =head2 new()
@@ -142,8 +65,6 @@ When making use of other key's values (as explainded in the example above) the e
 
 In some cases configuration files may need to include other configuration files.  The way this is done is via a speical key called 'include'. The same file will be automatically execluded if it's detected multiple times. 
 
-Command line arguments are captured and placed in the ARGV configuration section, command line arguments are B<CASE SENSITIVE>.
-
 =cut
 
 sub new {
@@ -166,7 +87,7 @@ By rule, entries outside of a section are 'global', entries within the CLI argum
 sub value {
 	my ($self, $sec, $key) = @_;
 	$sec = lc $sec;
-	$key = ($sec eq 'argv' ? $key : lc $key);
+	$key = lc $key;
 
 	if(ref($self->{$sec}->{$key}) eq 'ARRAY'){
 		return (@{ $self->{$sec}->{$key} });
@@ -241,50 +162,45 @@ sub _fileargs {
 
 			if(/^\s*?\[([^\]]+)\]\s*?$/){
 				$hd = lc $1;
-				next;
-			} elsif(!$hd){
+			} elsif(!$hd) {
 				$hd = $head || 'global';
 			}
 
 			s/^\s+//g;
 			s/\s+$//g;
 
-			# Skip blank lines and comments
 			next if(!$_ || /^#/);
 
-			# We use argv for our arguments string, and global for our global arguments,  and thus a sections named argv or global are disallowed.
+			# We use argv for our arguments string and thus a section named argv is disallowed.
 			if($hd eq 'argv'){
-				&_die("ERROR: section name [argv] is invalid and disallowed: $tfn\:$line" . ($fn ? ' included from: ' . join(' -> ', @{$fn}) : undef));
-			} elsif($hd eq 'global'){
-				&_die("ERROR: section name [global] is invalid and disallowed: $tfn\:$line" . ($fn ? ' included from: ' . join(' -> ', @{$fn}) : undef));
+				&_die("ERROR: section name [argv] is invalid and disallowed: $tfn\:$line" . ($fn ? ' included from: ' . join('-> ', @{$fn}) : undef));
 			}
 
+			if(/^([^\]]+)\s*?=(\s*?(.+?))?$/){
+				my ($k, $v) = ($1, $2);
 
-			my ($k, $v) = split(/=/, $_, 2);
+				$k =~ s/^\s+//g;
+				$k =~ s/\s+$//g;
+				$v =~ s/^\s+//g;
+				$v =~ s/\s+$//g;
 
-			$k =~ s/^\s+//g;
-			$k =~ s/\s+$//g;
-			$v =~ s/^\s+//g;
-			$v =~ s/\s+$//g;
+				if($k eq 'die'){
+					&_die("ERROR: $tfn died on line $line: $v in section [$hd] in $tfn" . ($fn ? ' included from: ' . join('-> ', @{$fn}) : undef));
+				} elsif(!$k){
+					next if !$k;
+				}
 
-			$k = lc $k;
-
-			if($k eq 'die'){
-				&_die("ERROR: $tfn died on line $line: $v in section [$hd] in $tfn" . ($fn ? ' included from: ' . join('-> ', @{$fn}) : undef));
-			} elsif(!$k){
-				next if !$k;
-			}
-
-			if(exists $conf->{$hd}->{$k} && $conf->{$hd}->{$k} ne "\x18"){
-				if(ref($conf->{$hd}->{$k}) eq 'ARRAY'){
-					push @{ $conf->{$hd}->{$k} }, $v;
+				if(exists $conf->{$hd}->{$k} && $conf->{$hd}->{$k} ne "\x18"){
+					if(ref($conf->{$hd}->{$k}) eq 'ARRAY'){
+						push @{ $conf->{$hd}->{$k} }, $v;
+					} else {
+						my $tmp = $conf->{$hd}->{$k};
+						delete $conf->{$hd}->{$k};
+						@{ $conf->{$hd}->{$k} } = ($tmp, $v);
+					}
 				} else {
-					my $tmp = $conf->{$hd}->{$k};
-					delete $conf->{$hd}->{$k};
-					@{ $conf->{$hd}->{$k} } = ($tmp, $v);
+					$conf->{$hd}->{$k} = $v;
 				}
-			} else {
-				$conf->{$hd}->{$k} = $v;
 			}
 		}
 		close($fh);
@@ -341,6 +257,7 @@ sub _cliargs {
 			$v =~ s/^\s+//g;
 			$v =~ s/\s+$//g;
 
+print "KV: $k $v\n";
 			if(exists $conf->{$k} && $conf->{$k} ne "\x18"){
 				if(ref($conf->{$k}) eq 'ARRAY'){
 					push @{ $conf->{$k} }, $v;
@@ -400,6 +317,38 @@ sub _cliargs {
 }
 
 
+# Combine results
+sub _stage1 {
+	my ($conf, $cfg) = @_;
+	if(ref($cfg) =~ /^(HASH|Config::Simple::Conf)$/){
+		for my $sec (keys %{ $cfg }){
+			for my $key (keys %{ $cfg->{$sec} }){
+				if(ref($conf->{$sec}->{$key}) eq 'ARRAY'){
+					if(ref($cfg->{$sec}->{$key}) eq 'ARRAY'){
+						push @{ $conf->{$sec}->{$key} }, @{ $cfg->{$sec}->{$key} };
+					} elsif($cfg->{$sec}->{$key}){
+						push @{ $conf->{$sec}->{$key} }, $cfg->{$sec}->{$key};
+					}
+				} else {
+					if(ref($cfg->{$sec}->{$key}) eq 'ARRAY'){
+						push @{ $cfg->{$sec}->{$key} }, $conf->{$sec}->{$key} if $conf->{$sec}->{$key};
+						$conf->{$sec}->{$key} = $cfg->{$sec}->{$key};
+					} else {
+						if($conf->{$sec}->{$key} && $cfg->{$sec}->{$key}){
+							push my @tmp, ($conf->{$sec}->{$key}, $cfg->{$sec}->{$key});
+							$conf->{$sec}->{$key} = \@tmp;
+						} elsif($cfg->{$sec}->{$key}){
+							$conf->{$sec}->{$key} = $cfg->{$sec}->{$key};
+						}
+					}
+				}
+			}
+		}
+	}
+
+	return $conf;
+}
+
 # This routine reprocesses the configuration hash, and replaces any [section:name], or [name] variables with their setting.
 sub _stage2 {
 	my ($conf) = @_;
@@ -414,33 +363,19 @@ sub _stage2 {
 						if($conf->{$sec}->{$key}->[$i] =~ /\[([^:]+)\:([^\]]+)]/){
 							my ($aa, $ab) = ($1, $2);
 
-							if($sec ne 'argv'){
-								$aa = lc $aa;
-								$ab = lc $ab;
-							}
-
 							if(ref($conf->{$aa}->{$ab}) eq 'ARRAY'){
 								&_die("ERROR: the embedded key [$aa:$ab] in section [$sec] must be unique: " . join("; ", @{ $conf->{$aa}->{$ab} }));
 							} elsif($conf->{$aa}->{$ab}){
-								my ($raa, $rab) = ($aa, $ab);
-								$raa =~ s/(\W)/\\$1/g;
-								$rab =~ s/(\W)/\\$1/g;
-
-								$conf->{$sec}->{$key}->[$i] =~ s/\[$raa:$rab\]/$conf->{$aa}->{$ab}/gi;
+								$conf->{$sec}->{$key}->[$i] =~ s/\[[^:]+\:[^\]]+]/$conf->{$aa}->{$ab}/g;
 								$try++;
 							}
 						} elsif($conf->{$sec}->{$key}->[$i] =~ /\[([^\]]+)]/){
 							my $aa = $1;
 
-							$aa = lc $aa if($sec ne 'argv');
-
 							if(ref($conf->{global}->{$aa}) eq 'ARRAY'){
 								&_die("ERROR: the embedded global key [$aa] in section [$sec] must be unique: " . join("; ", @{ $conf->{global}->{$aa} }));
 							} elsif($conf->{global}->{$aa}){
-								my $raa = $aa;
-								$raa =~ s/(\W)/\\$1/g;
-
-								$conf->{$sec}->{$key}->[$i] =~ s/\[$raa]/$conf->{global}->{$aa}/gi;
+								$conf->{$sec}->{$key}->[$i] =~ s/\[[^\]]+]/$conf->{global}->{$aa}/g;
 								$try++;
 							}
 						}
@@ -449,33 +384,19 @@ sub _stage2 {
 					if(!ref($conf->{$sec}->{$key}) && $conf->{$sec}->{$key} =~ /\[([^:]+):([^\]]+)\]/){
 						my ($aa, $ab) = ($1, $2);
 
-						if($sec ne 'argv'){
-							$aa = lc $aa;
-							$ab = lc $ab;
-						}
-
 						if(ref($conf->{$aa}->{$ab}) eq 'ARRAY'){
 							&_die("ERROR: the embedded key [$aa:$ab] in section [$sec] must be unique: " . join("; ", @{ $conf->{$aa}->{$ab} }));
 						} elsif($conf->{$aa}->{$ab}){
-							my ($raa, $rab) = ($aa, $ab);
-							$raa =~ s/(\W)/\\$1/g;
-							$rab =~ s/(\W)/\\$1/g;
-
-							$conf->{$sec}->{$key} =~ s/\[$raa:$rab\]/$conf->{$aa}->{$ab}/gi;
+							$conf->{$sec}->{$key} =~ s/\[[^:]+:[^\]]+\]/$conf->{$aa}->{$ab}/g;
 							$try++;
 						}
 					} elsif(!ref($conf->{$sec}->{$key}) && $conf->{$sec}->{$key} =~ /\[([^\]]+)\]/){
 						my $aa = $1;
 
-						$aa = lc $aa if($sec ne 'argv');
-
 						if(ref($conf->{global}->{$aa}) eq 'ARRAY'){
 							&_die("ERROR: the embedded key [$aa] in section [$sec] must be unique: " . join("; ", @{ $conf->{global}->{$aa} }));
 						} elsif($conf->{global}->{$aa}){
-							my $raa = $aa;
-							$raa =~ s/(\W)/\\$1/g;
-
-							$conf->{$sec}->{$key} =~ s/\[$raa\]/$conf->{global}->{$aa}/gi;
+							$conf->{$sec}->{$key} =~ s/\[[^\]]+\]/$conf->{global}->{$aa}/g;
 							$try++;
 						}
 					}
